local sti = require "libs.STI"
local bump = require 'libs/bump'
require("libs/AnAL")
require "classes/Player"
require "classes/Item"

function maketransparent(file)
   local imgdata = love.image.newImageData(file)
   for i=0,imgdata:getWidth()-1 do
      for j=0,imgdata:getHeight()-1 do
         local r,g,b,a = imgdata:getPixel(i,j)
         if r == 43 and g == 133 and b == 133 and a == 255 then
            imgdata:setPixel(i,j,255,255,255,0)
         end
      end
   end
   return love.graphics.newImage(imgdata)
end

function love.load()
	--print(sti.version)

	selectedP = "jake"
	selectedPClass = "Knight"
	player = Player:new(50,50,selectedP,selectedPClass)

	item = Item:new(50,155,"woodenBat",2)

	itemsOnMap = {
	}

	world = bump.newWorld(50)

	-- create two rectangles
	A = {name="A"}
	B = {name="B"}

	-- insert both rectangles into bump
	world:add(A,   player.x, player.y, 100, 100) -- left, top, width, height
	world:add(B, item.x, item.y, 100, 100)

	i		= 1
	speed	= 256

	-- Load multiple maps
	maps = {
		sti.new("assets/maps/rpgmap")
	}

	col = {}

	for k=1, #maps do
		local layer = 2
		showMenu = false
		menuSelectorPos = 184

		-- Add a Custom Layer
		maps[k]:addCustomLayer("Sprite Layer", layer)

		local spriteLayer = maps[k].layers["Sprite Layer"]
		local image = maketransparent(player.image)

		menuBG = love.graphics.newImage("assets/menu/menuBG.png")
		menuSelector = love.graphics.newImage("assets/menu/menuSelector.png");

		standing = newAnimation(image, 128, 192, 0.1, -1)
		standing:addFrame(0, 0, 32, 48, 0.1)

		walkLeft = newAnimation(image, 128, 192, 0.1, -1)
		walkLeft:addFrame(0, 48, 32, 48, 0.1)
		walkLeft:addFrame(32, 48, 32, 48, 0.1)
		walkLeft:addFrame(64, 48, 32, 48, 0.1)
		walkLeft:addFrame(96, 48, 32, 48, 0.1)

		walkRight = newAnimation(image, 128, 192, 0.1, -1)
		walkRight:addFrame(0, 96, 32, 48, 0.1)
		walkRight:addFrame(32, 96, 32, 48, 0.1)
		walkRight:addFrame(64, 96, 32, 48, 0.1)
		walkRight:addFrame(96, 96, 32, 48, 0.1)

		walkDown = newAnimation(image, 128, 192, 0.1, -1)
		walkDown:addFrame(0, 0, 32, 48, 0.1)
		walkDown:addFrame(32, 0, 32, 48, 0.1)
		walkDown:addFrame(64, 0, 32, 48, 0.1)
		walkDown:addFrame(96, 0, 32, 48, 0.1)

		walkUp = newAnimation(image, 128, 192, 0.1, -1)
		walkUp:addFrame(0, 144, 32, 48, 0.1)
		walkUp:addFrame(32, 144, 32, 48, 0.1)
		walkUp:addFrame(64, 144, 32, 48, 0.1)
		walkUp:addFrame(96, 144, 32, 48, 0.1)

		-- Add Custom Data
		spriteLayer.sprite = {
			--image = love.graphics.newImage("assets/sprites/"..sprite..".png"),  this draws the whole spritesheet, not what I need
			x = 495,
			y = 462,
			r = 0,
		}
	end
end

function love.update(dt)
	-- see if A is colliding with anything
	world:move(A, player.x, player.y)
	local collisions, len = world:check(B)

	if len > 0 then
		print("yay it works!")
	end

	s_w,s_h =  love.window.getDimensions( )
	-- Cap speed
	if speed < 128	then speed = 128 end
	if speed > 512	then speed = 512 end

	-- Update sprite's coordinates
	local sprite = maps[i].layers["Sprite Layer"].sprite
	down = love.keyboard.isDown

	if showMenu == false then
		if down("w") or down("up")	then
			item.y =  item.y + speed * dt
			sprite.y = sprite.y - speed * dt
			walkDown:stop()
			walkUp:play()
			walkUp:update(dt)
		end
		if down("s") or down("down") then
			item.y =  item.y - speed * dt
			sprite.y = sprite.y + speed * dt
			walkUp:stop()
			walkDown:play()
			walkDown:update(dt)
		end
		if down("a") or down("left")	then
			item.x =  item.x + speed * dt
			sprite.x = sprite.x - speed * dt
			walkRight:stop()
			walkLeft:play()
			walkLeft:update(dt)
		end
		if down("d") or down("right")	then
			item.x =  item.x - speed * dt
			sprite.x = sprite.x + speed * dt
			walkLeft:stop()
			walkRight:play()
			walkRight:update(dt)
		 end

		for _, map in pairs(maps) do
			map:update(dt)
		end
	end
end

function love.draw()

	local sprite = maps[i].layers["Sprite Layer"].sprite
	local ww = love.graphics.getWidth()
	local wh = love.graphics.getHeight()
	local tx = math.floor(-sprite.x + ww / 2 - 16)
	local ty = math.floor(-sprite.y + wh / 2 - 16)

	-- Draw sprite in centre of screen
	love.graphics.push()
	love.graphics.translate(tx, ty)
	maps[i]:setDrawRange(tx, ty, ww, wh)
	maps[i]:draw()
	--maps[i]:drawCollisionMap(col[i])
	love.graphics.pop()
	x = (s_w/2) - 16
	y = (s_h/2) - 24

	love.graphics.draw(item.image, item.x, item.y)

	if down("a") or down("left") then
		walkLeft:draw(x, y)
	end

	if down("d") or down("right") then
		walkRight:draw(x, y)
	end

	if down("s") or down("down") then
		walkDown:draw(x, y)
	end

	if down("w") or down("up") then
		walkUp:draw(x, y)
	end

	if ((down("a") or down("left"))==false) and ((down("d") or down("right"))==false) and ((down("s") or down("down"))==false) and ((down("w") or down("up"))==false) then
		standing:draw(x, y)
	end

	if showMenu == true then
		love.graphics.draw(menuBG, x-360, y-150)
		love.graphics.draw(menuSelector, x-295, menuSelectorPos)
	end

	love.graphics.setColor(255,255,255)
	love.graphics.rectangle("line",player.x, player.y, 100, 100)
	love.graphics.rectangle("line",item.x, item.y, 100, 100)
end

function love.keypressed(key, isrepeat)
--	if key == "lctrl"	then speed = speed - 64 end
--	if key == "lalt"	then speed = speed + 64 end
	if key == "m" then
		if showMenu == false then
			showMenu = true
		else
			showMenu = false
		end
	end
	if showMenu == true then
		if key == "down" and menuSelectorPos < 384 then
			menuSelectorPos = menuSelectorPos + 40
		end
		if key == "up" and menuSelectorPos ~= 184 then
			menuSelectorPos = menuSelectorPos - 40
		end
	end
end

function love.resize(w, h)
	for k=1, #maps do
		maps[k]:resize(w, h)
	end
end
